import os
from os.path import join, splitext, basename, dirname, isdir
from glob import glob
from typing import Dict

from ..util import replace_ext, gdal


def extract_subdataset_step(source_dir: os.PathLike, target_dir: os.PathLike, preprocessor_config: dict, subdataset_types: Dict[str, str]=None):
    filenames = []
    for dataglob in preprocessor_config.get('data_file_globs', '*'):
        for p in [path for path in glob(join(source_dir, '**', dataglob), recursive=True) if not isdir(path)]:
            filenames.append(p)
    if len(filenames) == 0:
        raise Exception('No datafiles were matched by the provided glob')

    for filename in datafiles:
        extract_subdatasets(
            filename,
            target_dir,
            subdataset_types
        )


def extract_subdatasets(source_filename: os.PathLike, target_dir: os.PathLike, subdataset_types: Dict[str, str]=None):
    ds = gdal.Open(source_filename)

    sub_datasets = []
    for locator, _ in ds.GetSubDatasets():
        _, _, sd_type = locator.split(':')
        if subdataset_types is None or sd_type in subdataset_types:
            sub_datasets.append((locator, subdataset_types[sd_type]))

    if not sub_datasets:
        raise Exception('No subdatasets were matched by the provided types')

    for locator, suffix in sub_datasets:
        target_filename = join(target_dir, basename(replace_ext(source_filename, '%s.tif' % suffix)))
        gdal.Translate(target_filename, locator, format='GTiff')
